unit Encoders;

{}
{						}
{       by Randy Beck                           }
{	    http://www.randybeck.com            }
{	    email:  rb@randybeck.com		}
{                                               }
{}

// This stream will perform a very simple encryption as it reads
// or writes the data.  The encryption is not world-class by any
// means, but it is not meant to be.  It was designed to allow
// programmers to quickly substitute a TEncryptionStream for a
// TFileStream, and not have to change anything else in their
// programs.  It's a quick enhancement that keeps most users from
// prying.

// You'll need to run this stream on your data.  Then replace any
// use of that file on TFileStream with TEncryptionStream.

// I recommend that programmers who use this should change the
// value of the global constant DefaultEncryptionCode.  And if
// you want something a little more secure, you can rewrite the
// Encode and Decode methods.

// You'll notice that the encryption is performed one byte at a
// time, and that this severely limits the degree of sophistication
// that may be possible.  But any serious method (like DES) would
// add more rules to how this stream may be used, and that wasn't
// my intent.

interface

uses
  Classes, SysUtils;

type
  TEncryptionStream = class(TFileStream)
  protected
    CodeBuf: PByteArray;
  public
    Code: byte;
    constructor Create(const FileName: string; Mode: word);
    destructor Destroy; override;
    function  Read(var Buffer; Count: longint) : longint; override;
    function  Write(const Buffer; Count: longint) : longint; override;
    function  Encode(AByte: byte) : byte;
    function  Decode(AByte: byte) : byte;
  end;

const
  DefaultEncryptionCode: byte = $36;

implementation

constructor TEncryptionStream.Create(const FileName: string; Mode: word);
begin
  inherited Create(FileName, Mode);
  Code := DefaultEncryptionCode;
  New(CodeBuf);
end;

destructor TEncryptionStream.Destroy;
begin
  Dispose(CodeBuf);
  CodeBuf := nil;
  inherited Destroy;
end;

function  TEncryptionStream.Read(var Buffer; Count: longint): longint;
var  i: longint;
     P: pointer;
begin
  P := @Buffer;
  Result := inherited Read(Buffer, Count);
  If Count > 0 then
    For i := 0 to pred(Count) do
    begin
      byte(P^) := Decode(byte(P^));
      Inc(integer(P));
    end;
end;

function  TEncryptionStream.Write(const Buffer; Count: longint): longint;
var  n: longint;
     P: pointer;

    procedure Encrypt(ACount: longint);
    var  i: integer;
    begin
      If ACount > 0 then For i := 0 to pred(ACount) do
      begin
        CodeBuf^[i] := Encode(byte(P^));
        Inc(integer(P));
      end;
    end;

begin
  n := 0;
  P := @Buffer;
  While Count > sizeof(CodeBuf^) do
  begin
    Encrypt(sizeof(CodeBuf^));
    n := n + inherited Write(CodeBuf^, sizeof(CodeBuf^));
    Dec(Count, sizeof(CodeBuf^));
  end;
  Encrypt(Count);
  Result := n + inherited Write(CodeBuf^, Count);
end;

function  TEncryptionStream.Encode(AByte: byte) : byte;
begin
  Result := AByte xor Code;
end;

function  TEncryptionStream.Decode(AByte: byte) : byte;
begin
  Result := AByte xor Code;
end;

end.

